import numpy as np

def getHarrisCorners(M, kappa, theta):
    # Compute Harris corners
    # Input:
    # M: structure tensor of shape (H, W, 2, 2)
    # kappa: float (parameter for Harris corner score) 
    # theta: float (threshold for corner detection)
    # Output:
    # score: numpy.ndarray (Harris corner score) of shape (H, W)
    # points: numpy.ndarray (detected corners) of shape (N, 2)

    ########################################################################
    # TODO:                                                                #
    # Compute the Harris corner score and find the corners.               #
    #                                                                      #
    # Hints:                                                               #
    # - The Harris corner score is computed using the determinant and      #
    #   trace of the structure tensor.                                     #
    # - Use the threshold theta to find the corners.                       #
    # - Use non-maximum suppression to find the corners.                   #
    ########################################################################

    # compute score using det and trace
    detM = M[:,:,0,0] * M[:,:,1,1] - M[:,:,1,0]**2

    traceM =  M[:,:,0,0] +  M[:,:,1,1]
    score = detM - kappa * traceM**2

    # you can disable non-maximum suppression (for debugging)
    disable_nms = False

    # padded image for easier non-max suppression check
    score_pad = -np.inf * np.ones((M.shape[0] + 2, M.shape[1] + 2))
    score_pad[1:-1, 1:-1] = score

    # find corners
    bool_harris = score > theta
    bool_nms = (score > score_pad[0:-2, 1:-1]) & (score > score_pad[2:, 1:-1]) & (score > score_pad[1:-1, 0:-2]) & (score > score_pad[1:-1, 2:])
    bool_nms = np.bitwise_or(bool_nms, disable_nms)
    y, x = np.where(np.bitwise_and(bool_harris, bool_nms))
    points = np.array([x, y]).T

    ########################################################################
    #                           END OF YOUR CODE                           #
    ########################################################################

    return score, points

